@@ -1,17 +1,14 @@
 /*
  * Wrappers around standard functions that use the heap.
  *
- * Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
- *
+ * Copyright (c) 2008
+ * 	Phil Thompson <phil@river-bank.demon.co.uk>
+ * 
  * This file is part of SIP.
- *
+ * 
  * This copy of SIP is licensed for use under the terms of the SIP License
  * Agreement.  See the file LICENSE for more details.
- *
- * This copy of SIP may also used under the terms of the GNU General Public
- * License v2 or v3 as published by the Free Software Foundation which can be
- * found in the files LICENSE-GPL2 and LICENSE-GPL3 included in this package.
- *
+ * 
  * SIP is supplied WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
@@ -31,6 +28,7 @@ static void nomem(void);
 /*
  * Wrap malloc() and handle any errors.
  */
+
 void *sipMalloc(size_t n)
 {
 	void *h;
@@ -38,29 +36,14 @@ void *sipMalloc(size_t n)
 	if ((h = malloc(n)) == NULL)
 		nomem();
 
-    memset(h, 0, n);
-
 	return h;
 }
 
 
-/*
- * Wrap calloc() and handle any errors.
- */
-void *sipCalloc(size_t nr, size_t n)
-{
-	void *h;
-
-	if ((h = calloc(nr, n)) == NULL)
-		nomem();
-
-    return h;
-}
-
-
 /*
  * Wrap strdup() and handle any errors.
  */
+
 char *sipStrdup(const char *s)
 {
 	char *h;
@@ -73,8 +56,9 @@ char *sipStrdup(const char *s)
 
 
 /*
- * Return a string on the heap which is the concatenation of all the arguments.
+ * Return a string on the heap which is the concatonation of all the arguments.
  */
+
 char *concat(const char *s, ...)
 {
 	const char *sp;
