@@ -1,7 +1,7 @@
 /*
  * The main header file for SIP.
  *
- * Copyright (c) 2007
+ * Copyright (c) 2008
  * 	Phil Thompson <phil@river-bank.demon.co.uk>
  * 
  * This file is part of SIP.
@@ -142,6 +142,7 @@
 #define CTOR_EXPLICIT       0x00000200  /* The ctor is explicit. */
 #define CTOR_CAST           0x00000400  /* The ctor is a cast. */
 #define CTOR_HOLD_GIL       0x00000800  /* The ctor holds the GIL. */
+#define CTOR_XFERRED        0x00001000  /* Ownership is transferred. */
 
 #define isPublicCtor(c)     ((c)->ctorflags & SECT_IS_PUBLIC)
 #define setIsPublicCtor(c)  ((c)->ctorflags |= SECT_IS_PUBLIC)
@@ -157,6 +158,8 @@
 #define isCastCtor(c)       ((c)->ctorflags & CTOR_CAST)
 #define isHoldGILCtor(c)    ((c)->ctorflags & CTOR_HOLD_GIL)
 #define setIsHoldGILCtor(c) ((c)->ctorflags |= CTOR_HOLD_GIL)
+#define isResultTransferredCtor(c)  ((c)->ctorflags & CTOR_XFERRED)
+#define setIsResultTransferredCtor(c)   ((c)->ctorflags |= CTOR_XFERRED)
 
 
 /* Handle member flags. */
@@ -209,6 +212,9 @@
 #define OVER_DONT_DEREF_SELF    0x00040000  /* For comparison operators, don't dereference self. */
 #define OVER_HOLD_GIL       0x00080000  /* The function holds the GIL. */
 #define OVER_RELEASE_GIL    0x00100000  /* The function releases the GIL. */
+#define OVER_THIS_XFERRED   0x00200000  /* Ownership of this is transferred. */
+#define OVER_IS_GLOBAL      0x00400000  /* It is a global operator. */
+#define OVER_IS_COMPLEMENTARY   0x00800000  /* It is a complementary operator. */
 
 #define isPublic(o)         ((o)->overflags & SECT_IS_PUBLIC)
 #define setIsPublic(o)      ((o)->overflags |= SECT_IS_PUBLIC)
@@ -223,6 +229,7 @@
 
 #define isVirtual(o)        ((o)->overflags & OVER_IS_VIRTUAL)
 #define setIsVirtual(o)     ((o)->overflags |= OVER_IS_VIRTUAL)
+#define resetIsVirtual(o)   ((o)->overflags &= ~OVER_IS_VIRTUAL)
 #define isAbstract(o)       ((o)->overflags & OVER_IS_ABSTRACT)
 #define setIsAbstract(o)    ((o)->overflags |= OVER_IS_ABSTRACT)
 #define isConst(o)          ((o)->overflags & OVER_IS_CONST)
@@ -248,6 +255,12 @@
 #define setIsHoldGIL(o)     ((o)->overflags |= OVER_HOLD_GIL)
 #define isReleaseGIL(o)     ((o)->overflags & OVER_RELEASE_GIL)
 #define setIsReleaseGIL(o)  ((o)->overflags |= OVER_RELEASE_GIL)
+#define isThisTransferredMeth(o)    ((o)->overflags & OVER_THIS_XFERRED)
+#define setIsThisTransferredMeth(o) ((o)->overflags |= OVER_THIS_XFERRED)
+#define isGlobal(o)         ((o)->overflags & OVER_IS_GLOBAL)
+#define setIsGlobal(o)      ((o)->overflags |= OVER_IS_GLOBAL)
+#define isComplementary(o)  ((o)->overflags & OVER_IS_COMPLEMENTARY)
+#define setIsComplementary(o)   ((o)->overflags |= OVER_IS_COMPLEMENTARY)
 
 
 /* Handle variable flags. */
@@ -275,6 +288,7 @@
 #define ARG_IN              0x0200  /* It passes an argument. */
 #define ARG_OUT             0x0400  /* It returns a result. */
 #define ARG_CONSTRAINED     0x0800  /* Suppress type conversion. */
+#define ARG_SINGLE_SHOT     0x1000  /* The slot is only ever fired once. */
 
 #define isReference(a)      ((a)->argflags & ARG_IS_REF)
 #define setIsReference(a)   ((a)-> argflags |= ARG_IS_REF)
@@ -303,6 +317,7 @@
 #define isConstrained(a)    ((a)->argflags & ARG_CONSTRAINED)
 #define setIsConstrained(a) ((a)->argflags |= ARG_CONSTRAINED)
 #define resetIsConstrained(a)   ((a)->argflags &= ~ARG_CONSTRAINED)
+#define isSingleShot(a)     ((a)->argflags & ARG_SINGLE_SHOT)
 
 
 /* Handle name flags. */
@@ -434,7 +449,8 @@ typedef enum {
     anyslot_type,
     cbool_type,
     sstring_type,
-    wstring_type
+    wstring_type,
+    fake_void_type
 } argType;
 
 
@@ -504,7 +520,7 @@ typedef struct _scopedNameDef {
 typedef struct _nameDef {
     int nameflags;                      /* The name flags. */
     struct _moduleDef *module;          /* The main module. */
-    char *text;                         /* The text of the name. */
+    const char *text;                   /* The text of the name. */
     struct _nameDef *next;              /* Next in the list. */
 } nameDef;
 
@@ -527,7 +543,7 @@ typedef struct _nodeDef {
 
 typedef struct _codeBlock {
     char *frag;                         /* The code itself. */
-    char *filename;                     /* The original file. */
+    const char *filename;               /* The original file. */
     int linenr;                         /* The line in the file. */
     struct _codeBlock *next;            /* Next in the list. */
 } codeBlock;
@@ -536,8 +552,8 @@ typedef struct _codeBlock {
 /* A module definition. */
 
 typedef struct _moduleDef {
-    char *fullname;                     /* The full module name. */
-    char *name;                         /* The module base name. */
+    const char *fullname;               /* The full module name. */
+    const char *name;                   /* The module base name. */
     int version;                        /* The module version. */
     int modflags;                       /* The module flags. */
     int qobjclass;                      /* QObject class, -1 if none. */
@@ -804,7 +820,6 @@ typedef struct _enumDef {
     nameDef *pyname;                    /* The Python name (may be NULL). */
     int enumnr;                         /* The enum number. */
     struct _classDef *ecd;              /* The enclosing class. */
-    struct _classDef *pcd;              /* The publishing class. */
     moduleDef *module;                  /* The owning module. */
     enumMemberDef *members;             /* The list of members. */
     struct _memberDef *slots;           /* The list of slots. */
@@ -865,7 +880,7 @@ typedef struct _classDef {
     int classflags;                     /* The class flags. */
     int userflags;                      /* The user type flags. */
     int classnr;                        /* The class number. */
-    char *pyname;                       /* The Python name. */
+    const char *pyname;                 /* The Python name. */
     ifaceFileDef *iff;                  /* The interface file. */
     struct _classDef *ecd;              /* The enclosing scope. */
     struct _classDef *real;             /* The real class if this is a proxy or extender. */
@@ -973,10 +988,10 @@ void fatal(char *,...);
 void fatalScopedName(scopedNameDef *);
 int setInputFile(FILE *open_fp, parserContext *pc, int optional);
 void *sipMalloc(size_t);
-char *sipStrdup(char *);
+char *sipStrdup(const char *);
 char *concat(const char *, ...);
-void append(char **,char *);
-ifaceFileList *addToUsedList(ifaceFileList **, ifaceFileDef *);
+void append(char **, const char *);
+void addToUsedList(ifaceFileList **, ifaceFileDef *);
 int excludedFeature(stringList *,qualDef *);
 int sameSignature(signatureDef *,signatureDef *,int);
 int sameTemplateSignature(signatureDef *sd1, signatureDef *sd2, int deep);
@@ -992,7 +1007,7 @@ void appendCodeBlock(codeBlock **headp, codeBlock *new);
 void prcode(FILE *fp, const char *fmt, ...);
 void prOverloadName(FILE *fp, overDef *od);
 void prScopedPythonName(FILE *fp, classDef *scope, const char *pyname);
-void prOverloadDecl(FILE *fp, overDef *od, int defval);
+void prOverloadDecl(FILE *fp, classDef *context, overDef *od, int defval);
 void searchTypedefs(sipSpec *pt, scopedNameDef *snd, argDef *ad);
 int isIntReturnSlot(memberDef *md);
 int isLongReturnSlot(memberDef *md);
@@ -1008,6 +1023,7 @@ int optNoEmitters(sipSpec *pt);
 int optRegisterTypes(sipSpec *pt);
 int optQ_OBJECT4(sipSpec *pt);
 void yywarning(char *);
+nameDef *cacheName(sipSpec *pt, const char *name);
 
 
 /* These are only here because bison publically references them. */
