@@ -2,7 +2,7 @@
  * This module implements a hash table class for mapping C/C++ addresses to the
  * corresponding wrapped Python object.
  *
- * Copyright (c) 2007
+ * Copyright (c) 2008
  * 	Phil Thompson <phil@river-bank.demon.co.uk>
  * 
  * This file is part of SIP.
@@ -109,6 +109,16 @@ sipWrapper *sipOMFindObject(sipObjectMap *om, void *key, sipWrapperType *type)
     /* Go through each wrapped object at this address. */
     for (w = he->first; w != NULL; w = w->next)
     {
+        /*
+         * If the reference count is 0 then it is in the process of being
+         * deleted, so ignore it.  It's not completely clear how this can
+         * happen (but it can) because it implies that the garbage collection
+         * code is being re-entered (and there are guards in place to prevent
+         * this).
+         */
+        if (w->ob_refcnt == 0)
+            continue;
+
         /*
          * If this wrapped object is of the given type, or a sub-type of it,
          * then we assume it is the same C++ object.
@@ -250,13 +260,12 @@ int sipOMRemoveObject(sipObjectMap *om,sipWrapper *val)
             *wp = val -> next;
 
             /*
-             * If the bucket is now empty then count it as stale.
-             * Note that we do not NULL the key and count it as
-             * unused because that might throw out the search for
-             * another entry that wanted to go here, found it
-             * already occupied, and was put somewhere else.  In
-             * other words, searches must be repeatable until we
-             * reorganise the table.
+             * If the bucket is now empty then count it as stale.  Note that we
+             * do not NULL the key and count it as unused because that might
+             * throw out the search for another entry that wanted to go here,
+             * found it already occupied, and was put somewhere else.  In other
+             * words, searches must be repeatable until we reorganise the
+             * table.
              */
             if (he -> first == NULL)
                 om -> stale++;
