@@ -1,7 +1,7 @@
 /*
  * This file defines the SIP library internal interfaces.
  *
- * Copyright (c) 2007
+ * Copyright (c) 2008
  * 	Phil Thompson <phil@river-bank.demon.co.uk>
  * 
  * This file is part of SIP.
@@ -59,8 +59,6 @@ extern PyInterpreterState *sipInterpreter;  /* The interpreter. */
 extern sipQtAPI *sipQtSupport;  /* The Qt support API. */
 extern sipWrapperType *sipQObjectClass; /* The Python QObject class. */
 
-void *sip_api_convert_rx(sipWrapper *txSelf, const char *sigargs,
-        PyObject *rxObj, const char *slot, const char **memberp);
 void *sipGetRx(sipWrapper *txSelf, const char *sigargs, PyObject *rxObj,
         const char *slot, const char **memberp);
 int sip_api_emit_signal(PyObject *self, const char *sig, PyObject *sigargs);
@@ -100,6 +98,8 @@ void sipSaveMethod(sipPyMethod *pm,PyObject *meth);
 void *sipGetPending(sipWrapper **op, int *fp);
 PyObject *sipWrapSimpleInstance(void *cppPtr, sipWrapperType *type,
         sipWrapper *owner, int initflags);
+void *sipConvertRxEx(sipWrapper *txSelf, const char *sigargs,
+        PyObject *rxObj, const char *slot, const char **memberp, int flags);
 
 void sipOMInit(sipObjectMap *om);
 void sipOMFinalise(sipObjectMap *om);
