@@ -2,7 +2,7 @@
 # extension modules created with SIP.  It provides information about file
 # locations, version numbers etc., and provides some classes and functions.
 #
-# Copyright (c) 2007
+# Copyright (c) 2008
 # 	Phil Thompson <phil@river-bank.demon.co.uk>
 # 
 # This file is part of SIP.
@@ -1231,11 +1231,16 @@ class PythonModuleMakefile(Makefile):
         tail = dirname[len(self._moddir):]
 
         flist = []
-        for f in names:
+        for f in list(names):
             # Ignore certain files.
             if f in ("Makefile", ):
                 continue
 
+            # Do not recurse into certain directories.
+            if f in (".svn", "CVS"):
+                names.remove(f)
+                continue
+
             if os.path.isfile(os.path.join(dirname, f)):
                 flist.append(os.path.join(self._srcdir + tail, f))
 
